#    MIDAS Access Server  for  Spectrum 

global env
source [file join $env(MIDASBASE) TclHttpd tcl SpectrumDomainHandler.tcl]       ;#  load default support procedures

global MIDAS_SpectrumAccessServer_Verbose
set    MIDAS_SpectrumAccessServer_Verbose 1

global MIDAS_SOAPDomainHandler_Verbose
set MIDAS_SOAPDomainHandler_Verbose 0

package require Spectrum

# Use namespaces to isolate your methods
namespace eval urn:SpectrumServer {
  catch {namespace import ::Spectrum::Debug}
  catch {namespace import ::Spectrum::Logging}
  catch {namespace import ::Spectrum::New1D}
  catch {namespace import ::Spectrum::New2D}
  catch {namespace import ::Spectrum::Delete}
  catch {namespace import ::Spectrum::Zero}
  catch {namespace import ::Spectrum::Read1DR}
  catch {namespace import ::Spectrum::Read1D}
  catch {namespace import ::Spectrum::Read2D}
  catch {namespace import ::Spectrum::Write1D}
  catch {namespace import ::Spectrum::Write2D}
  catch {namespace import ::Spectrum::Compressions}
  catch {namespace import ::Spectrum::Error}

  catch {namespace import ::base64::encode}
  catch {namespace import ::base64::decode}

  set SpectrumIndex 1

  proc SpectrumServerVerbose {option} {
        global MIDAS_SpectrumAccessServer_Verbose
        global MIDAS_SOAPDomainHandler_Verbose
        set MIDAS_SpectrumAccessServer_Verbose $option
        set MIDAS_SOAPDomainHandler_Verbose $option
        set z [catch {Logging $option} m]
        if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecVerbose detected error in call to Spectrum::Logging"}

        set status "$m"
        return [rpcvar::rpcvar string $status]
  }

    proc SpecDebug {args} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecDebug with parameter $args"}
        set z [catch {Debug $args} m]
        if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecDebug detected error in call to Spectrum::Debug"}

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecLogging {args} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecLogging with parameter $args"}
        set z [catch {Logging $args} m]
        if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecLogging detected error in call to Spectrum::Logging"}

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecNew {Spectrum b r args} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecNew with parameters $Spectrum $b $r $args,(len: [llength $args])"}
        if {[llength $args]==0 || [second $args]==0} {
          set z [catch {LocalSpecNew1D $Spectrum $b $r} m]
          if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecNew detected error in call to Spectrum::New1D"}
        } else {
          set z [catch {LocalSpecNew2D $Spectrum $b $r [first $args] [second $args]} m]
          if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecNew detected error in call to Spectrum::New2D"}
        }

        return $m
    }

    proc SpecNew1D {Spectrum b r} {
        return [LocalSpecNew1D $Spectrum $b $r]
    }

    proc SpecNew2D {Spectrum b0 r0 b1 r1} {
        return [LocalSpecNew2D $Spectrum $b0 $r0 $b1 $r1]
    }

    proc LocalSpecNew1D {Spectrum b r} {
        global   MIDAS_SpectrumAccessServer_Verbose
        upvar 2 SpectrumIndex SpectrumIndex
        upvar 2 Spectra Spectra

        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecNew1D with parameters $Spectrum $b $r"}

#    Check Spectrum name is not already in use
        if {[info exists Spectra($Spectrum)]} then {
             return -code error -errorcode "0x30004" -errorinfo "Spectrum name already in use"
        }

        set Spectra($Spectrum) $SpectrumIndex

        upvar 2 S$SpectrumIndex S
        set S(dim) 1
        set S(base_0) $b
        set S(range_0) $r

        incr SpectrumIndex

        set z [catch {New1D $Spectrum $b $r} m]
        if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecNew1D detected error in call to Spectrum::New1D"}

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc LocalSpecNew2D {Spectrum b0 r0 b1 r1} {
        global   MIDAS_SpectrumAccessServer_Verbose
        upvar 2 SpectrumIndex SpectrumIndex
        upvar 2 Spectra Spectra

        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecNew2D with parameters $Spectrum $b0 $r0 $b1 $r1"}

#    Check Spectrum name is not already in use
        if {[info exists Spectra($Spectrum)]} then {
             return -code error -errorcode "0x30004" -errorinfo "Spectrum name already in use"
        }

        set Spectra($Spectrum) $SpectrumIndex

        upvar 2 S$SpectrumIndex S
        set S(dim) 2
        set S(base_0) $b0
        set S(range_0) $r0
        set S(base_1) $b1
        set S(range_1) $r1

        incr SpectrumIndex

        set z [catch {New2D $Spectrum $b0 $r0 $b1 $r1} m]
        if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecNew2D detected error in call to Spectrum::New2D"}

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecDelete {Spectrum} {
        global   MIDAS_SpectrumAccessServer_Verbose
        upvar Spectra Spectra

        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecDelete with parameter $Spectrum"}

        if {![info exists Spectra($Spectrum)]} then {
             return -code error -errorcode 0x30004 -errorinfo "Spectrum name does not exist"
        }

        set Index $Spectra($Spectrum)
        upvar S$Index S
        array unset S

        unset Spectra($Spectrum)

        set z [catch {Delete $Spectrum} m]
        if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecDelete detected error in call to Spectrum::Delete"}

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecZero {Spectrum} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecZero with parameter $Spectrum"}
        set z [catch {Zero $Spectrum} m]
        if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecZero detected error in call to Spectrum::Zero"}

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecDetails {Spectrum} {
        global   MIDAS_SpectrumAccessServer_Verbose
        upvar Spectra Spectra

        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecDetails with parameters $Spectrum"}

#    Check Spectrum name is in use
        if {![info exists Spectra($Spectrum)]} then {
             return -code error -errorcode "0x30004" -errorinfo "Invalid spectrum name"
        }

        set Index $Spectra($Spectrum)
        upvar S$Index S
        set m {}
        set d $S(dim)
        lappend m $d

        set b {}
        for {set i 0} {$i < $d} {incr i} {
           lappend b $S(base_$i)
        }
        lappend m $b

        set r {}
        for {set i 0} {$i < $d} {incr i} {
           lappend r $S(range_$i)
        }
        lappend m $r

        lappend m {0 0} {5 5} ; # for backwards compatibility

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecNames {pattern} {
        global   MIDAS_SpectrumAccessServer_Verbose
        upvar Spectra Spectra

        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecNames with parameter $pattern"}

        if {![info exists Spectra]} then {
             return -code error -errorcode 0x30004 -errorinfo "Spectrum names do not exist"
        }

	if {[llength $pattern] == 1} {set pattern [first $pattern]}
	if {[llength $pattern] == 0} {set pattern "*"}
        set m {}
        foreach Spectrum [array names Spectra -glob $pattern] {
           lappend m [list "1" "$Spectrum"]
        }

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecRead1D {Spectrum args} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecRead1D with parameters $Spectrum $args"}
        if {[llength $args] < 2} {
           return -code error -errorcode "0x30005" -errorinfo "SpecRead1D: not enough parameters"
        } else {
           set b [lindex $args 0]
           set r [lindex $args 1]
        }
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "calling method Spectrum::Read1D with parameters $Spectrum $b $r"}
        set z [catch {Read1D $Spectrum $b $r} d]
        if {$z != 0} then {return -code error -errorcode "$d" -errorinfo "SpecRead1D detected error in call to Spectrum::Read1D"} \
        else {
           set z [catch {encode $d} m]
           if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecRead1D detected error in call to ::base64::encode"}
        }

        set status "$m"
        return [rpcvar::rpcvar base64Binary $status]
    }

    proc SpecRead1DR {Spectrum args} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecRead1DR with parameters $Spectrum $args"}
        if {[llength $args] < 2} {
           return -code error -errorcode "0x30005" -errorinfo "SpecRead1DR: not enough parameters"
        } else {
           set b [lindex $args 0]
           set r [lindex $args 1]
        }
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "calling method Spectrum::Read1DR with parameters $Spectrum $b $r"}
        set z [catch {Read1DR $Spectrum $b $r} m]
        if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecRead1DR detected error in call to Spectrum::Read1DR"}

        set status "$m"
        return [rpcvar::rpcvar base64Binary $status]
    }

    proc SpecRead2D {Spectrum args} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecRead2D with parameters $Spectrum $args"}
        if {[llength $args] < 4} {
           return -code error -errorcode "0x30005" -errorinfo "SpecRead2D: not enough parameters"
        } else {
           set b1 [lindex $args 0]
           set r1 [lindex $args 1]
           set b2 [lindex $args 2]
           set r2 [lindex $args 3]
        }
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "calling method Spectrum::Read2D with parameters $Spectrum $b1 $r1 $b2 $r2"}
        set z [catch {Read2D $Spectrum $b1 $r1 $b2 $r2} d]
        if {$z != 0} then {return -code error -errorcode "$d" -errorinfo "SpecRead2D detected error in call to Spectrum::Read2D"} \
        else {
           set z [catch {encode $d} m]
           if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecRead2D detected error in call to ::base64::encode"}
        }

        set status "$m"
        return [rpcvar::rpcvar base64Binary $status]
    }

     proc SpecWrite1D {Spectrum c args} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecWrite1D with parameters $Spectrum (counts) $args"}
        if {$MIDAS_SpectrumAccessServer_Verbose && [string length $c] < 256} {puts "counts $c"}
        if {[llength $args] < 2} {
           return -code error -errorcode "0x30005" -errorinfo "SpecWrite1D: not enough parameters"
        } else {
           set b [lindex $args 0]
           set r [lindex $args 1]
        }
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "calling method Spectrum::Write1D with parameters $Spectrum $b $r (counts)"}
        set z [catch {decode $c} d]
        if {!$z} {set z [catch {Write1D $Spectrum $b $r $d} m]
        } else {set m $d}

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecWrite2D {Spectrum c args} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecWrite2D with parameters $Spectrum (counts) $args"}
        if {$MIDAS_SpectrumAccessServer_Verbose && [string length $c] < 256} {puts "counts $c"}
        if {[llength $args] < 4} {
           return -code error -errorcode "0x30005" -errorinfo "SpecWrite2D: not enough parameters"
        } else {
           set b1 [lindex $args 0]
           set r1 [lindex $args 1]
           set b2 [lindex $args 2]
           set r2 [lindex $args 3]
        }
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "calling method Spectrum::Write2D with parameters $Spectrum $b1 $r1 $b2 $r2 (counts)"}
        set z [catch {decode $c} d]
        if {!$z} {set z [catch {Write2D $Spectrum $b1 $r1 $b2 $r2 $d} m]
        } else {set m $d}

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecGetTitle {Spectrum} {
        global   MIDAS_SpectrumAccessServer_Verbose
        upvar Spectra Spectra

        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecGetTitle with parameters $Spectrum"}

#    Check Spectrum name is in use
        if {![info exists Spectra($Spectrum)]} then {
             return -code error -errorcode "0x30004" -errorinfo "Invalid spectrum name"
        }

        set Index $Spectra($Spectrum)
        upvar S$Index S
        if {[info exists S(info_1)]} then {
           set m $S(info_1)
        } else {
           set m ""
        }

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecPutTitle {Spectrum t} {
        global   MIDAS_SpectrumAccessServer_Verbose
        upvar Spectra Spectra

        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecPutTitle with parameters $Spectrum $t"}

#    Check Spectrum name is in use
        if {![info exists Spectra($Spectrum)]} then {
             return -code error -errorcode "0x30004" -errorinfo "Invalid spectrum name"
        }

        set Index $Spectra($Spectrum)
        upvar S$Index S
        set S(info_1) $t

        return [rpcvar::rpcvar string ""]
    }

    proc SpecGetLabel {Spectrum} {
        global   MIDAS_SpectrumAccessServer_Verbose
        upvar Spectra Spectra

        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecGetLabel with parameters $Spectrum"}

#    Check Spectrum name is in use
        if {![info exists Spectra($Spectrum)]} then {
             return -code error -errorcode "0x30004" -errorinfo "Invalid spectrum name"
        }

        set Index $Spectra($Spectrum)
        upvar S$Index S
        if {[info exists S(info_2)]} then {
           set m $S(info_2)
        } else {
           set m ""
        }

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecPutLabel {Spectrum l} {
        global   MIDAS_SpectrumAccessServer_Verbose
        upvar Spectra Spectra

        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecPutLabel with parameters $Spectrum $l"}

#    Check Spectrum name is in use
        if {![info exists Spectra($Spectrum)]} then {
             return -code error -errorcode "0x30004" -errorinfo "Invalid spectrum name"
        }

        set Index $Spectra($Spectrum)
        upvar S$Index S
        set S(info_1) $t

        return [rpcvar::rpcvar string ""]
    }

    proc SpecGetInfo {Spectrum n} {
        global   MIDAS_SpectrumAccessServer_Verbose
        upvar Spectra Spectra

        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecGetInfo with parameters $Spectrum $n"}

#    Check Spectrum name is in use
        if {![info exists Spectra($Spectrum)]} then {
             return -code error -errorcode "0x30004" -errorinfo "Invalid spectrum name"
        }

        set Index $Spectra($Spectrum)
        upvar S$Index S
        if {[info exists S(info_$n)]} then {
           set m $S(info_$n)
        } else {
           set m ""
        }

        set status "$m"
        return [rpcvar::rpcvar string $status]

    }

    proc SpecPutInfo {Spectrum n i} {
        global   MIDAS_SpectrumAccessServer_Verbose
        upvar Spectra Spectra

        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecPutInfo with parameters $Spectrum $n $i"}

#    Check Spectrum name is in use
        if {![info exists Spectra($Spectrum)]} then {
             return -code error -errorcode "0x30004" -errorinfo "Invalid spectrum name"
        }

        set Index $Spectra($Spectrum)
        upvar S$Index S
        set S(info_$n) $i

        return [rpcvar::rpcvar string ""]
    }

    proc SpecGetCalib {Spectrum d} {
        global   MIDAS_SpectrumAccessServer_Verbose
        upvar Spectra Spectra

        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecGetCalib with parameters $Spectrum $d"}

#    Check Spectrum name is in use
        if {![info exists Spectra($Spectrum)]} then {
             return -code error -errorcode "0x30004" -errorinfo "Invalid spectrum name"
        }

        set Index $Spectra($Spectrum)
        upvar S$Index S
        if {$d > $S(dim)} then {
             return -code error -errorcode "0x30005" -errorinfo "Calibration string number out of range"
        }
        if {[info exists S(calib_$d)]} then {
           set m $S(calib_$d)
        } else {
           set m ""
        }

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecPutCalib {Spectrum d c} {
        global   MIDAS_SpectrumAccessServer_Verbose
        upvar Spectra Spectra

        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecPutCalib with parameters $Spectrum $d $c"}

#    Check Spectrum name is in use
        if {![info exists Spectra($Spectrum)]} then {
             return -code error -errorcode "0x30004" -errorinfo "Invalid spectrum name"
        }

        set Index $Spectra($Spectrum)
        upvar S$Index S
        if {$d > $S(dim)} then {
             return -code error -errorcode "0x30005" -errorinfo "Calibration string number out of range"
        }
        set S(calib_$d) $c

        return [rpcvar::rpcvar string ""]
    }

    proc SpecCompressions {c1 c2} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecCompressions with parameters $c1 $c2"}
        set z [catch {Compressions $c1 $c2} m]

        if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecCompressions detected error in call to Spectrum::Compressions"}

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecError {e} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecError with parameter $e"}
        set z [catch {Error $e} m]

        if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecError detected error in call to Spectrum::Error"}

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }


    # We have to publish the public methods...

     SOAP::export SpectrumServerVerbose
     SOAP::export SpecDebug
     SOAP::export SpecLogging
     SOAP::export SpecNew
     SOAP::export SpecNew1D
     SOAP::export SpecNew2D
     SOAP::export SpecDelete
     SOAP::export SpecZero
     SOAP::export SpecDetails
     SOAP::export SpecNames
     SOAP::export SpecRead1D
     SOAP::export SpecRead1DR
     SOAP::export SpecRead2D
     SOAP::export SpecWrite1D
     SOAP::export SpecWrite2D
     SOAP::export SpecGetTitle
     SOAP::export SpecPutTitle
     SOAP::export SpecGetLabel
     SOAP::export SpecPutLabel
     SOAP::export SpecGetInfo
     SOAP::export SpecPutInfo
     SOAP::export SpecGetCalib
     SOAP::export SpecPutCalib
     SOAP::export SpecCompressions
     SOAP::export SpecError
}

# register this service with tclhttpd
puts [SOAP::Domain::register  -prefix  /SpectrumServer -namespace urn:SpectrumServer -uri urn:SpectrumServer]


proc ::SOAP::Domain::MIDAS_SOAPDomainHandler_/SpectrumServer {optsname sock args} {   \
    ::SOAP::Domain::MIDAS_SOAPDomainHandler_/DefaultAccessServer $optsname $sock $args}

package provide SpectrumServer 1.0


#       set z [catch {interp eval "" ::namespace inscope $options(-namespace) $methodName $argValues} msg]
#       if {[llength $msg]==2} {
#          append xml "<status xsi:type=\"xsd:[lindex [set msg0 [lindex $msg 0]] 4]\">[lindex $msg0 5]</status>"
#          append xml "<return xsi:type=\"xsd:[lindex [set msg1 [lindex $msg 1]] 4]\">[lindex $msg1 5]</return>"
#       } else {
#          append xml "<return xsi:type=\"xsd:[lindex $msg 4]\">[lindex $msg 5]</return>"
#       }

